-----------------------
-------VARIABLES-------
-----------------------


local ADDON_NAME, namespace = ...;
local variables = namespace.variables;
local functions = namespace.functions;

local elapsed = 0;


-----------------------
------EVENT TABLE------
-----------------------


variables.events = 
{
    ["ENCOUNTER_LOOT_RECEIVED"] = 
    function(...)
        local _, _, item, _, name = ...;
        
        if name ~= variables.playerName then
            return;
        end
		
        elapsed = GetTime();

        C_Timer.After(1, 
            function()
                for bag = 0, NUM_BAG_SLOTS do
                    for slot = 1, GetContainerNumSlots(bag) do
                        local bagLink = GetContainerItemLink(bag, slot);
            
                        if bagLink and bagLink == item then
                            if functions.scanToolTipAndQueue(bag, slot, bagLink) then 
                                return;
                            end
                        end
                    end
                end
            end
        );
    end,
    ["BAG_UPDATE"] = 
    function(...)
        if elapsed > 0 then
            print("Time between events: " .. (GetTime() - elapsed));

            elapsed = 0;
        end
    end,
    ["GROUP_ROSTER_UPDATE"] = 
    function(...)
        functions.toggleLootListener();
        functions.toggleWhisperListener();
        functions.controlRollListener();
    end,
    ["CHAT_MSG_RAID_WARNING"] = 
    function(...)
        local msg, author = ...;
        local author = strsplit("-", author);
        
        --Check if the person sending a raidwarning is the raid leader or if assist setting is enabled.
        if not functions.isLeaderOrAssistWithMode(author) then
            return;
        end
        
        local _, link, _, _, _, _, _, _, itemEquipLoc, _, _, itemClassID, itemSubClassID = GetItemInfo(msg);
        local isPet = itemClassID == 15 and itemSubClassID == 2;

        if not 
        (
            itemClassID == 4 or --Armor
            itemClassID == 2 or --Weapon
            isPet 				--Pet
        )
        then
            return;
        end

        local itemValues = functions.getItemValues(link);
        local usableStats = functions.usableStats(itemValues);
        local anyStats = itemValues.agility or itemValues.strength or itemValues.intellect;

        local usabilityVar = 
        --Check if it is a pet.
        isPet or
        --Check if it item is a cloak.
        itemEquipLoc == "INVTYPE_CLOAK" or 
        --Check if it is a ring.
        itemEquipLoc == "INVTYPE_FINGER" or 
        --Check if it is a neck.
        itemEquipLoc == "INVTYPE_NECK" or 
        --Check if it is a trinket, if the trinket has any main stats then use that to detect if usable.
        (itemEquipLoc == "INVTYPE_TRINKET" and ((anyStats and usableStats) or not anyStats)) or
        --Check if item is an off-hand, and if player should use off-hands.
        (itemEquipLoc == "INVTYPE_HOLDABLE" and variables.classArray[variables.playerClassId][itemEquipLoc]) or
        --Check if item is an equippable weapon/armor piece with correct stats.
        (variables.classArray[variables.playerClassId][itemClassID][itemSubClassID] and usableStats) or 
        --Check if item has mog that user can get.
        functions.usableMog(link);

        local ownerName = select(3, msg:find("(.+) (- .+)"));
        local owner = 
        (
            UnitPlayerOrPetInRaid(ownerName) 
            and 
            {
                name = ownerName,
                color = functions.getClassColor(ownerName)
            }
            or 
            nil
        );

        if usabilityVar then
            C_ChatInfo.SendAddonMessage("LPTLootRoll", "potentialRollers-itemUsable", "RAID");
        end

        if UnitIsGroupLeader("player") or llrSettings.leadMode then
            functions.historyLengthControl();

            --Insert the new item into the rollhistory array.
            tinsert(
                variables.rollHistory, 
                1, 
                {
                    item 		     = link,
                    mainList 	     = {},
                    offList 	     = {},
                    mogList 	     = {},
                    otherList        = {},
                    distributed      = false,
                    owner            = owner,
                    potentialRollers = {}
                }
            );

            functions.leaderEvent(usabilityVar);
        elseif usabilityVar then
            functions.userEvent(link, owner);
        end
    end,
    ["CHAT_MSG_SYSTEM"] = 
    function(...)
        if not variables.rollHistory[1] or variables.rollHistory[1].distributed then
			return;
		end

		local msg = ...;
		local author, roll, rollMin, rollMax = strmatch(msg, "(.+) rolls (%d+) %((%d+)-(%d+)%)");
		
		if not author then
			return;
		end
		
		--Convert the rolls to numbers.
		roll 	= tonumber(roll);
		rollMin = tonumber(rollMin);
        rollMax = tonumber(rollMax);

        --Make sure the values were found and that it does not exceed the maximum allowed value.
		if not roll or not rollMax or rollMin ~= 1 or rollMax > variables.maxRollValue then 
			return;
		end

        --Determine what array the roller should be added to.
        local determinedArray = variables.rollHistory[1].otherList;
        local rollType = "other";

		if rollMax == llrSettings.mainRoll then
            determinedArray = variables.rollHistory[1].mainList;
            rollType = "main";
		elseif rollMax == llrSettings.offRoll then
			determinedArray = variables.rollHistory[1].offList;
            rollType = "off";
		elseif rollMax == llrSettings.mogRoll then
			determinedArray = variables.rollHistory[1].mogList;
            rollType = "transmog";
		else
			roll = rollMax;
        end
        
		--Make sure the user has not already rolled.
		if #determinedArray > 0 then
			for _, v in pairs(determinedArray) do 
				if v[1] == author then
					return;
				end
			end
        end
		
		--Insert the roller into the array.
        tinsert(
            determinedArray, 
            {
                author, 
                roll, 
                functions.getClassColor(author), 
                rollType
            }
        );

        variables.rollHistory[1].potentialRollers[author] = true;
        functions.updateRollerCounter();
	
        --Sort table based primarly by roll number, secondly by name.
        table.sort(
            determinedArray, 
            function(a, b)
                if a[2] > b[2] then
                    return true;
                elseif a[2] < b[2] then
                    return false;
                else
                    return a[1] < b[1];
                end
            end
        );
    end,
    ["CHAT_MSG_WHISPER"] = 
    function(...)
        local msg, author = ...;
        local name, _ = strsplit("-", author);

        if UnitPlayerOrPetInRaid(name) then
            functions.registerItem(name, msg);
        end
    end,
    ["CHAT_MSG_ADDON"] =
    function(...)
        local prefix, msg, _, sender = ...;

        if prefix ~= "LPTLootRoll" then
            return;
        end

        local key, value = strsplit("-", msg);

        if key == "potentialRollers" and variables.rollHistory[1] ~= nil and not variables.rollHistory[1].distributed then
            local name, realm = strsplit("-", sender);

            if value == "itemUsable" then
                if #variables.rollHistory[1].potentialRollers > 0 then
                    for i, v in pairs(variables.rollHistory[1].potentialRollers) do
                        if i == name then
                            return;
                        end
                    end
                end
            
                variables.rollHistory[1].potentialRollers[name] = false;
            elseif value == "itemPassed" and not variables.rollHistory[1].potentialRollers[name] then
                variables.rollHistory[1].potentialRollers[name] = true;
            end

            functions.updateRollerCounter();
        elseif key == "itemDistributed" then
            --Hide user window and disable leader roll buttons when an item has been marked as distributed.
            functions.hideUser();
            functions.disableLeader();
        end
    end
};